/*
 * lc7981.h
 *
 *  Created on: 01.05.2009
 *      Author: sebastian
 *
 *    This file is part of Sebastians AVR Library for lc7981.
 *
 *   Sebastians AVR Library is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Sebastians AVR Library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Sebastians AVR Library.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *			'Do the thing you want to do and let your spirits guide you through.
 *			Live a life, worth being remembered.'
 *			-- Remembered - Firewind --
 */




#ifndef LC7981_H_
#define LC7981_H_ LC7981_H_ 	//!< protects this file from beeing double included

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "portsmacro.h"

#define LCD_CTRL_PORT 		D		//!< Port used for control signals
#define LCD_DATA_PORT		B
#define BACKLIGHT_PORT		A


#define LCD_CTRL 		PORT(LCD_CTRL_PORT)	//!< Port used for control signals
#define LCD_CRTL_DDR		DDR(LCD_CTRL_PORT)	//!< Data-Direction-Register for the control signals
#define LCD_RS			2		//!< Register-Select Pin of the Display
#define LCD_RW			3		//!< Read-Write-select Pin
#define LCD_EN			4		//!< Strobe Pin
#define	LCD_RESET		7
#define LCD_CS			6

#define BACKLIGHT_EN		0
#define BACKLIGHT		PORT(BACKLIGHT_PORT)
#define BACKLIGHT_DDR		DDR(BACKLIGHT_PORT)


#define BUSYBIT			7
#define LCD_DATA		PORT(LCD_DATA_PORT)	//!< Port used for data
#define LCD_DATA_PIN		PIN(LCD_DATA_PORT)	//!< Port used for data
#define LCD_DATA_DDR		DDR(LCD_DATA_PORT)	//!< Data-Direction-Register for Data

// Macros for (un)setting the control pins
#define lcd_rs_instruction() (LCD_CTRL |= (1 << LCD_RS))//!< Set the Register-Select pin high
#define lcd_rs_data() (LCD_CTRL &= ~(1 << LCD_RS))	//!< Set the Register-Select pin low

#define lcd_rw_read() (LCD_CTRL |= (1 << LCD_RW))	//!< Set the Read-Write-Select pin high
#define lcd_rw_write() (LCD_CTRL &= ~(1 << LCD_RW))	//!< Set the Register-Select pin low

#define lcd_en_active() (LCD_CTRL |= (1 << LCD_EN))	//!< Set the strobe pin high
#define lcd_en_passive() (LCD_CTRL &= ~(1 << LCD_EN))	//!< Set the strobe pin low


#define LCD_TEXT_MODE 	  	0		//!< Constant for text mode
#define LCD_TEXT_LINES 	 	10 		//!< Number of lines in text mode
#define LCD_TEXT_COLUMNS 	26		//!< Number of columns in text mode

#define LCD_GRAPHIC_MODE 	1		//!< Constant for graphic mode
#define LCD_GRAPHIC_WIDTH 	160		//!< Horizontal display size in pixels
#define LCD_GRAPHIC_HEIGHT 	80		//!< Vertical display size in pixels
#define PIXEL_ON 	    	1		//!< see lcd_plot_pixel
#define PIXEL_OFF 	    	0		//!< see lcd_plot_pixel

void lcd_init(uint8_t mode);
void lcd_clear(void);

void lcd_write_text(char *txt);
void lcd_write_text_P(const char *txt);
void lcd_gotoxy(uint8_t x, uint8_t y);

void lcd_plot_pixel(uint8_t x, uint8_t y, uint8_t set);
void lcd_plot_bitmap(uint8_t x, uint8_t y, const uint8_t *bitmap, uint8_t w, uint8_t h);

void lcd_plot_char(uint8_t x_off, uint8_t y_off, uint8_t c, uint8_t fw, uint8_t fh, const uint8_t* font);
void lcd_plot_text(uint8_t x_off, uint8_t y_off, const char *text, uint8_t fw, uint8_t fh, const uint8_t *font);
void lcd_plot_text_P(uint8_t x_off, uint8_t y_off, const char *text, uint8_t fw, uint8_t fh, const uint8_t *font);


//void lcd_write_command(uint8_t cmd, uint8_t data);
uint8_t lcd_read_byte(void);

#define lcd_setpixel(x,y)	lcd_plot_pixel(x,y,PIXEL_ON) 
void lcd_line (uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);

#define DISPLAY_ON()	BACKLIGHT |= (1<<BACKLIGHT_EN)
#define DISPLAY_OFF()	BACKLIGHT &= ~(1<<BACKLIGHT_EN)

#define LCD_DIR_READ()  LCD_DATA_DDR = 0
#define LCD_DIR_WRITE()	LCD_DATA_DDR = 0xff


#endif /* LC7981_H_ */

