/**
 * writing_demo.c
 *
 *  Created on: 29.05.2009
 *      Author: sebastian
 */

#include <stdlib.h>
#include <avr/eeprom.h>
#include <avr/sleep.h>

#include "writing_demo.h"
#include "8x8_horizontal_LSB_1.h"

#define DEBUG_AD	0
#define IDLETIME	30

extern const uint8_t bitmap[] PROGMEM;

uint8_t ee_dummy[4]  EEMEM;
uint16_t ee_x_offset EEMEM;
uint16_t ee_x_mul    EEMEM;
uint16_t ee_x_div    EEMEM;
uint16_t ee_y_offset EEMEM;
uint16_t ee_y_mul    EEMEM;
uint16_t ee_y_div    EEMEM;

void draw_marker(uint8_t x, uint8_t y)
{
	lcd_line (x-5, y-5, x+5, y+5);
	lcd_line (x-5, y+5, x+5, y-5);
}

#define P1X	(LCD_GRAPHIC_WIDTH/10)
#define P1Y	(LCD_GRAPHIC_HEIGHT/10)
#define P2X	(LCD_GRAPHIC_WIDTH-(LCD_GRAPHIC_WIDTH/10))
#define	P2Y	(LCD_GRAPHIC_HEIGHT-(LCD_GRAPHIC_HEIGHT/10))

#define DELTA_X	(P2X-P1X)
#define DELTA_Y (P2Y-P1Y)

extern int16_t x_offset, x_mul, x_div;
extern int16_t y_offset, y_mul, y_div;
extern const uint8_t font_12x16[] PROGMEM;

void calibrate_touch(void)
{
	int16_t p1x, p1y, p2x, p2y;
#if DEBUG_AD
	static char bff[40];
#endif
	x_offset = eeprom_read_word(&ee_x_offset);
	x_mul    = eeprom_read_word(&ee_x_mul);
	x_div    = eeprom_read_word(&ee_x_div);
	y_offset = eeprom_read_word(&ee_y_offset);
	y_mul    = eeprom_read_word(&ee_y_mul);
	y_div    = eeprom_read_word(&ee_y_div);

	touch_debounce_off();

	lcd_clear();
	lcd_plot_bitmap(20, 20, bitmap, 32, 32);
	lcd_plot_text_P(28,22,PSTR("OK"),8,8,font_8x8);
	touch_debounce_on();
	while(-1 == (p1x = touch_readX()))
		;	
	while(-1 == (p1y = touch_readY()))
		;
	touch_debounce_off();

	if((p1x >= 20 && p1x <= 52) && (p1y >= 20 && p1y <= 52))
	{
		return;
	}
	/* Else we need to recalibrate */
	x_offset = 0;
	x_mul = x_div = 1;
	y_offset = 0;
	y_mul = y_div = 1;

	/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
	lcd_clear();
	lcd_plot_text_P(2,20,PSTR("Bitte die Mitte tippen"),8,8,font_8x8);
	draw_marker(P1X,P1Y);
	touch_debounce_on();
	while(-1 == (p1x = touch_readX()))
		;	
	while(-1 == (p1y = touch_readY()))
		;
	touch_debounce_off();
	lcd_clear();
	lcd_plot_text_P(2,20,PSTR("Bitte die Mitte tippen"),8,8,font_8x8);
	draw_marker(P2X,P2Y);
	touch_debounce_on();
	while(-1 == (p2x = touch_readX()))
		;	
	while(-1 == (p2y = touch_readY()))
		;
	touch_debounce_off();
	x_offset = p1x - ((p2x - p1x + 4))/8;		/* Berechnete Spannung fuer X=0 */
	x_mul = 32000/(((p2x - p1x)*10 + 4)/8);  	/* Skalierungsfaktor für volle Breite == 32000 */
	x_div = (32000/LCD_GRAPHIC_WIDTH);

	y_offset = p1y - ((p2y - p1y + 4))/8;		/* Berechnete Spannung fuer Y=0 */
	y_mul = 32000/(((p2y - p1y)*10 + 4)/8);  	/* Skalierungsfaktor für volle Hoehe == 32000 */
	y_div = (32000/LCD_GRAPHIC_HEIGHT);

	lcd_clear();
#if DEBUG_AD
	itoa(p1x, bff, 10);
	lcd_plot_text(8,8,bff,8,8,font_8x8);
	itoa(p1y, bff, 10);
	lcd_plot_text(88,8,bff,8,8,font_8x8);
	itoa(p2x, bff, 10);
	lcd_plot_text(8,40,bff,8,8,font_8x8);
	itoa(p2y, bff, 10);
	lcd_plot_text(88,40,bff,8,8,font_8x8);
	touch_debounce_on();
#endif
	eeprom_write_word(&ee_x_offset, x_offset);
	eeprom_write_word(&ee_x_mul, x_mul);
	eeprom_write_word(&ee_x_div, x_div);
	eeprom_write_word(&ee_y_offset, y_offset);
	eeprom_write_word(&ee_y_mul, y_mul);
	eeprom_write_word(&ee_y_div, y_div);
	touch_debounce_off();
}


void writing_demo(void) 
{
	uint8_t i;
	int16_t x,y;
	uint32_t idle = 0;

	do
	{
	    lcd_clear();
	    lcd_plot_bitmap(0, 0, bitmap, 32, 32);

//	    set_sleep_mode(SLEEP_MODE_IDLE);
	    set_sleep_mode(SLEEP_MODE_ADC);
	    while(1) 
	    {
		for(i = 0; i < 10; i++)
		{
		   cli();
	           sleep_enable();
        	   sei();
        	   sleep_cpu();
        	   sleep_disable();
  		   if(touch_is_pressed()) break;
		}

		if(touch_is_pressed()) 
		{
		    if(idle >= 1000*IDLETIME)
		    {
			DISPLAY_ON();
			touch_debounce_off();
			idle = 0;
		    }
		    else
		    {
			idle = 0;
			x = touch_readX();
			y = touch_readY();
			if(x != -1 && y != -1)
			{
			    if(x < 32 && y < 32)
				break;
			    lcd_plot_pixel(x-1,y+1,PIXEL_ON);
			    lcd_plot_pixel(x,y+1,PIXEL_ON);
			    lcd_plot_pixel(x+1,y+1,PIXEL_ON);
			    lcd_plot_pixel(x-1,y,PIXEL_ON);
			    lcd_plot_pixel(x,y,PIXEL_ON);
			    lcd_plot_pixel(x+1,y,PIXEL_ON);
			    lcd_plot_pixel(x-1,y-1,PIXEL_ON);
			    lcd_plot_pixel(x,y-1,PIXEL_ON);
			    lcd_plot_pixel(x+1,y-1,PIXEL_ON);
			}
		    }
		}
		else if(++idle > 1000*IDLETIME)
		{
			--idle;
			DISPLAY_OFF();
			do
			{
			   cli();
		           sleep_enable();
	        	   sei();
	        	   sleep_cpu();
	        	   sleep_disable();
			} while(!touch_is_pressed());
		}
	    }
	} while(1);
}

